import sys

args = sys.argv[1:]

if args:
    from bibleit import command, context

    if result := command.eval(context.Context(), *args):
        print(result)
else:
    from bibleit import repl

    repl.run()



import sys

from operator import attrgetter as _attrgetter

from bibleit import config as _config
from bibleit.bible import Bible as _Bible
from bibleit import screen as _screen

_FLAGS_ON = ["true", "on"]
_FLAGS_OFF = ["false", "off"]
_FLAGS_TOGGLE = _FLAGS_ON + _FLAGS_OFF

_FLAGS = {"debug", "color", "label", "screen"}


def _flag(value):
    if value := value.lower():
        assert (
            value in _FLAGS_TOGGLE
        ), f"value must be a boolean value: <{'|'.join(_FLAGS_TOGGLE)}>"
        return value in _FLAGS_ON
    return False


for _flag_name in _FLAGS:

    def _flag_method(name):
        fn = lambda ctx, value: setattr(_config, name, _flag(value))
        fn.__doc__ = f"Configure {_flag_name} config\n\n    set {_flag_name} <{'|'.join(_FLAGS_TOGGLE)}>"
        return fn

    setattr(sys.modules[__name__], _flag_name, _flag_method(_flag_name))


def bible(ctx, *args):
    """Configure bible one or more translation

    set bible <translation[, translation[, ...]]>

    Examples:
        set bible kjv
        set bible acf, nvi/pt"""
    translations = [value for arg in args for value in arg.split(",") if value]
    ctx.bible = sorted(
        {_Bible(translation.lower()) for translation in translations},
        key=_attrgetter("version"),
    )
    _config.label = len(translations) > 1


import curses
import types

ESC_KEY = 27
EXIT_KEYS = [ord('q'), ord('Q'), ESC_KEY]


def init(ctx, result):
    ctx.screen = types.SimpleNamespace()
    ctx.screen.result = result
    ctx.screen.stdscr = curses.initscr()
    ctx.screen.stdscr.clear()
    curses.noecho()
    curses.cbreak()
    ctx.screen.stdscr.keypad(True)
    ctx.screen.stdscr.scrollok(True)
    curses.curs_set(0)
    ctx.screen.selected_idx = 0
    ctx.screen.max_y, ctx.screen.max_x = ctx.screen.stdscr.getmaxyx()
    ctx.screen.scroll_start = 0
    ctx.screen.visible_lines = (ctx.screen.max_y - 2) // max(curses.curs_set(0), 1)

    
def display(ctx):
    if ctx.screen.result:
        output = list(filter(None, ctx.screen.result.splitlines()))
        ctx.screen.total_lines = len(output)

        display_lines = [
            line
            for line in output[ctx.screen.scroll_start:]
            if line.strip()
        ][:ctx.screen.visible_lines]

        if ctx.screen.selected_idx < ctx.screen.scroll_start:
            ctx.screen.selected_idx = ctx.screen.scroll_start
        elif ctx.screen.selected_idx >= ctx.screen.scroll_start + ctx.screen.visible_lines:
            ctx.screen.selected_idx = ctx.screen.scroll_start + ctx.screen.visible_lines - 1

        ctx.screen.stdscr.clear()
        for i, line in enumerate(display_lines):
            if i == ctx.screen.selected_idx - ctx.screen.scroll_start:
                ctx.screen.stdscr.addstr(i, 0, line, curses.A_REVERSE)
            else:
                ctx.screen.stdscr.addstr(line)
            ctx.screen.stdscr.addstr("\n")

        key = ctx.screen.stdscr.getch()

        if key == curses.KEY_UP:
            if ctx.screen.selected_idx > 0:
                ctx.screen.selected_idx -= 1
                if ctx.screen.selected_idx < ctx.screen.scroll_start:
                    ctx.screen.scroll_start = ctx.screen.selected_idx
        elif key == curses.KEY_DOWN:
            if ctx.screen.selected_idx < ctx.screen.total_lines - 1:
                if (
                    ctx.screen.selected_idx - ctx.screen.scroll_start >= ctx.screen.visible_lines - 1
                    or ctx.screen.selected_idx == len(display_lines) - 1
                ):
                    ctx.screen.scroll_start += 1
                ctx.screen.selected_idx = min(ctx.screen.selected_idx + 1, ctx.screen.total_lines - 1)
        elif key == curses.KEY_PPAGE:
            ctx.screen.selected_idx = max(0, ctx.screen.selected_idx - ctx.screen.visible_lines)
            ctx.screen.scroll_start = max(0, ctx.screen.scroll_start - ctx.screen.visible_lines)
        elif key == curses.KEY_NPAGE:
            ctx.screen.selected_idx = min(ctx.screen.total_lines - 1, ctx.screen.selected_idx + ctx.screen.visible_lines)
            ctx.screen.scroll_start = min(ctx.screen.scroll_start + ctx.screen.visible_lines, ctx.screen.total_lines - ctx.screen.visible_lines)

        if key in EXIT_KEYS:
            close(ctx)
    else:
        close(ctx)


def close(ctx):
    if ctx.screen:
        ctx.screen = None
    curses.echo()
    curses.nocbreak()
    curses.endwin()


import itertools
import functools
import re
import importlib.resources

from bibleit import config as _config
from bibleit import translations as _translations
from bibleit import normalize


_COLOR_END = "\x1b[0m"
_COLOR_LEN = 255 // len(_config.available_bible)
_VERSE_SLICE_DELIMITER = ":"
_VERSE_RANGE_DELIMITER = "-"
_VERSE_CONTINUATION_DELIMITER = "+"
_VERSE_CONTINUATION_DEFAULT = f"1{_VERSE_CONTINUATION_DELIMITER}"
_VERSE_POINTER_DELIMITER = "^"
_SEARCH_MULTIPLE_WORDS_DELIMITER = "+"
_TRANSLATIONS_DIR = importlib.resources.files(_translations)


class BibleNotFound(AssertionError):
    def __init__(self, version) -> None:
        super().__init__(f"Bible translation not found: {version}")
        self.version = version


class BibleMeta(type):
    def __init__(cls, name, bases, attrs):
        super().__init__(name, bases, attrs)
        cls._ids = itertools.count(1)


class Bible(metaclass=BibleMeta):
    def __init__(self, version):
        self.id = next(self.__class__._ids)
        self.version = version.lower()
        self.color = "\x1b[48;2;{};{};{}m".format(
            self.id + _COLOR_LEN,
            (self.id + 2) * _COLOR_LEN,
            (self.id + 3) * _COLOR_LEN,
        )
        try:
            target = _TRANSLATIONS_DIR / self.version
            if not target.is_file():
                raise BibleNotFound(self.version)
            with target.open() as f:
                self.content = list(
                    enumerate(
                        (line.strip(), normalize.normalize(line)) for line in f
                    )
                )
            self.display = functools.reduce(
                lambda f, g: lambda x: f(g(x)),
                [self.labeled, self.colored],
                lambda x: x,
            )
        except ValueError as e:
            raise BibleNotFound(self.version) from e

    def __repr__(self):
        return self.colored(self.version)

    def __hash__(self):
        return hash(self.version)

    def __eq__(self, other):
        return self.__class__ == other.__class__ and self.version == other.version

    def labeled(self, value):
        return f"({self.version}) {value}" if _config.label else value

    def colored(self, value):
        return (
            "{}{}{}".format(self.color, value, _COLOR_END) if _config.color else value
        )

    def book(self, name):
        return [
            line
            for line, (_, normalized) in self.content
            if re.search(rf"^{name}", normalized, re.IGNORECASE)
        ]

    def chapter(self, book, name):
        return [
            line
            for line, (_, normalized) in self.content
            if re.search(rf"^{book}.* {name}:", normalized, re.IGNORECASE)
        ]

    def chapters(self):
        names = {
            self.display(line[: re.search(r"\d+:\d+", line).start() - 1]): None
            for n, (line, _) in self.content
        }
        return names.keys()

    def ref(self, book, ref):
        target_ref = normalize.normalize_ref(ref).split(_VERSE_SLICE_DELIMITER)
        match target_ref:
            case [chapter, *verses]:
                verse = "".join(verses) or _VERSE_CONTINUATION_DEFAULT
                if verse.endswith(_VERSE_CONTINUATION_DELIMITER):
                    verse = int(verse[: verse.index(_VERSE_CONTINUATION_DELIMITER)]) - 1
                    return self.chapter(book, int(chapter))[verse:]
                match verse.split(_VERSE_RANGE_DELIMITER):
                    case [start]:
                        start, before = self._versePointer(start)
                        return self.chapter(book, int(chapter))[
                            int(start) - (before + 1) : int(start)
                        ]
                    case [start, end]:
                        start, before = self._versePointer(start)
                        end, after = self._versePointer(end)
                        return self.chapter(book, int(chapter))[
                            int(start) - (before + 1) : end + after
                        ]
            case []:
                return self.book(book)

    def _filter(self, *values):
        return [
            (line, normalized)
            for line, normalized in self.content
            if all(
                re.search(rf"\b{value}s??\b", normalized, re.IGNORECASE)
                for value in values
            )
        ]

    def _versePointer(self, value):
        if _VERSE_POINTER_DELIMITER in value:
            return map(
                int, map(lambda p: p or "0", value.split(_VERSE_POINTER_DELIMITER))
            )
        return int(value), 0

    def search(self, value):
        return [
            self.display(line)
            for line, _ in self._filter(*value.split(_SEARCH_MULTIPLE_WORDS_DELIMITER))
        ]

    def count(self, value):
        if target := value.lower():
            return sum(
                normalized.count(target)
                for _, normalized in self._filter(value)
            )
        return 0

    def refs(self, args):
        target = None
        match args:
            case [number, book, ref] if number.isdigit():
                target = self.ref(f"{number} {book}", ref)
            case [book, ref]:
                if book and book.isdigit():
                    target = self.book(f"{book} {ref}")
                target = self.ref(book, ref)
            case [book]:
                target = self.book(book)
        return target

    def ref_parse(self, args):
        return [self.display(self.content[line][1][0]) for line in args]



import sys
import traceback
from importlib import import_module

from bibleit import config, normalize
from bibleit.command import core

_prefix = "bibleit.command"
_default_module = "core"


def eval_methods(module):
    return {
        name
        for name in dir(module)
        if not name.startswith("_")
        if name not in sys.builtin_module_names
    }


def eval_module(name):
    target = f"{_prefix}.{name}"

    try:
        if target not in sys.modules:
            return import_module(target)
        return sys.modules[target]
    except ModuleNotFoundError as e:
        if config.debug:
            print(e)
        return eval_module(_default_module)


def eval(ctx, *line, module=None):
    """Evaluate a function with arbitrary arguments."""
    if module is None:
        module = _default_module
    try:
        name, *args = line

        if name.startswith("!"):
            args.insert(0, name[1:])
            name = "set"

        ctx.module = eval_module(module)
        ctx.methods = eval_methods(ctx.module)

        args = list(filter(None, args))
        
        target = f"{'{} '.format(module) if module != _default_module else ''}{name}{' {}'.format(' '.join(args)) if args else ''}"
        if name in ctx.methods:
            fn = getattr(ctx.module, name)
            nargs = [normalize.normalize(arg) for arg in args]
            return fn(ctx, *nargs)
    except AssertionError as e:
        print(f"Error: {e}")
    except Exception as e:
        if config.debug:
            print(f"*** {e}\n")
            print(traceback.format_exc())
        if module != _default_module:
            core.help(ctx, *[module, *line])
        else:
            core.help(ctx, name)
    else:
        print(f"Error: command '{target}' not found")



import importlib.resources
import bibleit.translations as _translations

# General
debug = False
color = False
label = False
screen = False
application = "bibleit"
version = "0.0.11"
help = 'Type "help" for more information.'
welcome = f"""
    Welcome to {application} v{version}

    {help}
"""

# Bible
translation_dir = "translations"
available_bible = importlib.resources.contents(_translations)
default_bible = "nvi"

# Repl
history_length = 1000
context_ps1 = ">"



import argparse

def get_name(translation):
	return f"diff.{translation}.chapters.count"

def diff(a, b):
	na, nb = map(get_name, [a,b])
	with open(na) as ba, open(nb) as bb:
		for x, y in zip(ba, bb):
			xx, yy = x.split("$")[1].strip(), y.split("$")[1].strip()
			if xx != yy:
				print(x.strip(), y.strip())


if __name__ == "__main__":
	parser = argparse.ArgumentParser(description='Bible differences')
	parser.add_argument('a', help='left')
	parser.add_argument('b', help='right')

	args = parser.parse_args()

	diff(args.a, args.b)